/**-----------------------------------------------------------------------------------------
* Copyright © 2025 Progress Software Corporation. All rights reserved.
* Licensed under commercial license. See LICENSE.md in the project root for more information
*-------------------------------------------------------------------------------------------*/
import { TemplateRef, QueryList } from '@angular/core';
import { CellTemplateDirective } from '../rendering/cell-template.directive';
import { EditTemplateDirective } from '../editing/edit-template.directive';
import { ColumnBase } from './column-base';
import { ColumnComponent } from "./column.component";
import { OptionChangesService } from '../common/option-changes.service';
import * as i0 from "@angular/core";
/**
 * @hidden
 */
export declare function isSpanColumnComponent(column: any): column is SpanColumnComponent;
/**
 * Represents a column that spans multiple data cells while keeping individual header and footer cells.
 * Use this component to create flexible layouts and retain built-in UI for [sorting]({% slug sorting_treelist %}) and [filtering]({% slug filtering_treelist %}).
 * Wrap the columns to merge inside the `<kendo-treelist-span-column>` tag.
 *
 * @example
 * ```html
 * <kendo-treelist-span-column>
 *   <kendo-treelist-column field="field1"></kendo-treelist-column>
 *   <kendo-treelist-column field="field2"></kendo-treelist-column>
 *   <ng-template kendoTreeListCellTemplate let-dataItem>
 *     <h5>{{ dataItem.field1 }}</h5>
 *     <p>{{ dataItem.field2 }}</p>
 *   </ng-template>
 * </kendo-treelist-span-column>
 * ```
 */
export declare class SpanColumnComponent extends ColumnBase {
    /**
     * Specifies if the expanded indicator appears in the column.
     */
    expandable: boolean;
    readonly isSpanColumn: boolean;
    /**
     * @hidden
     */
    title: string;
    /**
     * @hidden
     */
    headerClass: string | string[] | Set<string> | {
        [key: string]: any;
    };
    /**
     * @hidden
     */
    footerClass: string | string[] | Set<string> | {
        [key: string]: any;
    };
    /**
     * @hidden
     */
    headerStyle: {
        [key: string]: string;
    };
    /**
     * @hidden
     */
    footerStyle: {
        [key: string]: string;
    };
    template: QueryList<CellTemplateDirective>;
    editTemplate: QueryList<EditTemplateDirective>;
    /**
     * @hidden
     */
    childColumns: QueryList<ColumnComponent>;
    /**
     * @hidden
     */
    includeInChooser: boolean;
    /**
     * Defines whether the edit template of the column is rendered.
     * To enable editing for a spanned column, set an edit template for it.
     * @default false
     */
    set editable(value: boolean);
    get editable(): boolean;
    /**
     * @hidden
     * added for backwards compitability
     */
    set width(_value: number);
    get width(): number;
    /**
     * @hidden
     */
    get leafIndex(): number;
    private _editable;
    constructor(parent?: ColumnBase, optionChanges?: OptionChangesService);
    /**
     * @hidden
     */
    get templateRef(): TemplateRef<any>;
    /**
     * @hidden
     */
    get editTemplateRef(): TemplateRef<any>;
    /**
     * @hidden
     */
    get colspan(): number;
    /**
     * Toggles the locked (frozen) state of the columns. Locked columns are visible at all times during horizontal scrolling. [See example](slug:locked_columns_treelist).
     * @default false
     */
    set locked(value: boolean);
    get locked(): boolean;
    get isEditable(): boolean;
    get childrenArray(): any[];
    get hasChildren(): boolean;
    static ɵfac: i0.ɵɵFactoryDeclaration<SpanColumnComponent, [{ optional: true; host: true; skipSelf: true; }, null]>;
    static ɵcmp: i0.ɵɵComponentDeclaration<SpanColumnComponent, "kendo-treelist-span-column", never, { "expandable": { "alias": "expandable"; "required": false; }; "title": { "alias": "title"; "required": false; }; "headerClass": { "alias": "headerClass"; "required": false; }; "footerClass": { "alias": "footerClass"; "required": false; }; "headerStyle": { "alias": "headerStyle"; "required": false; }; "footerStyle": { "alias": "footerStyle"; "required": false; }; "editable": { "alias": "editable"; "required": false; }; "locked": { "alias": "locked"; "required": false; }; }, {}, ["template", "editTemplate", "childColumns"], never, true, never>;
}
